package cz.mendelu.dnaAnalyser.analyse.g4killer;

import cz.mendelu.dnaAnalyser.sequence.SequenceType;
import cz.mendelu.dnaAnalyser.sequence.data.SequenceData;
import cz.mendelu.dnaAnalyser.utils.controller.PostMapping;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiOperation;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import javax.validation.constraints.Pattern;
import java.nio.ByteBuffer;

@RestController
@Api(tags = "api/analyse/g4killer")
public class G4KillerController {

    @Data
    static class AnalyzeSequenceRequest {
        @Pattern(regexp = "[ACTGactg]+",
                message = "Input must be sequence of ACTG")
        @ApiModelProperty(
                example = "GGACATGCCCGGGCATGTCC",
                notes = "DNA sequence")
        private String sequence;
        private float threshold;
        private Boolean onComplementary;
    }

    @Autowired
    private G4KillerService g4KillerService;

    @PostMapping("/api/analyse/g4killer")
    @PreAuthorize("isAuthenticated()")
    @ApiOperation("Execute g4killer instant analyse ")
    @ResponseStatus(HttpStatus.CREATED)
    public Outcome analyzeSequence(
            @RequestBody @Valid AnalyzeSequenceRequest analyzeSequenceRequest) {
        return g4KillerService.sequenceAnalyse(
                SequenceData.wrap(ByteBuffer.wrap(analyzeSequenceRequest.sequence.toUpperCase().getBytes()), false, SequenceType.DNA),
                analyzeSequenceRequest.threshold,
                analyzeSequenceRequest.onComplementary);
    }
}
