package cz.mendelu.dnaAnalyser.analyse.p53predictor;

import cz.mendelu.dnaAnalyser.sequence.stream.BufferedWindow;
import cz.mendelu.dnaAnalyser.sequence.stream.Window;
import cz.mendelu.dnaAnalyser.utils.controller.Response;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiOperation;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import javax.validation.constraints.Pattern;

@RestController
@Api(tags = "api/analyse/p53predictor")
@RequestMapping("api/analyse")
public class P53PredictorController {

    @Autowired
    private P53PredictorAnalyse p53PredictorAnalyse;

    /**
     * Class for tool analyse request.
     */
    @Data static class P53PredictorToolRequest {

        @Pattern(regexp = "[ACTG]{20}",
                message = "Input must be sequence of ACTG with exact length 20.")
        @ApiModelProperty(
                example = "GGACATGCCCGGGCATGTCC",
                notes = "DNA sequence with length 20.")
        private String sequence;
    }

    @PostMapping("/p53predictor/tool")
    @ResponseStatus(HttpStatus.OK)
    @PreAuthorize("isAuthenticated()")
    @ApiOperation("Create simple analyse for sequence length 20 from input with no database effects.")
    public Response<P53PredictorResult> p53predictorTool(
            @RequestBody @Valid P53PredictorToolRequest request)
    {
        Window sequenceWindow = BufferedWindow.wrap(request.sequence);
        P53PredictorResult result = p53PredictorAnalyse.calcP53PredictorScore(sequenceWindow);
        return Response.response(result);
    }
}
